@@ -30,7 +30,6 @@ import org.apache.maven.model.Resource;
 import org.apache.maven.plugin.*;
 import org.apache.maven.project.MavenProject;
 import org.codehaus.plexus.util.StringUtils;
-import org.osgi.service.cm.ConfigurationAdmin;
 
 /**
  * The <code>SCRDescriptorMojo</code>
@@ -79,6 +78,13 @@ public class SCRDescriptorMojo extends AbstractMojo {
      */
     private boolean generateAccessors;
 
+    /**
+     * The comma separated list of tokens to exclude when processing sources.
+     *
+     * @parameter alias="excludes"
+     */
+    private String sourceExcludes;
+
     /**
      * @see org.apache.maven.plugin.AbstractMojo#execute()
      */
@@ -89,9 +95,11 @@ public class SCRDescriptorMojo extends AbstractMojo {
         boolean hasFailures = false;
 
         JavaClassDescriptorManager jManager = new JavaClassDescriptorManager(this.getLog(),
-                                                                             this.project);
+                                                                             this.project,
+                                                                             this.sourceExcludes);
         // iterate through all source classes and check for component tag
         final JavaClassDescription[] javaSources = jManager.getSourceDescriptions();
+        Arrays.sort(javaSources, new JavaClassDescriptionInheritanceComparator());
 
         final Components components = new Components();
         final Components abstractComponents = new Components();
@@ -113,6 +121,7 @@ public class SCRDescriptorMojo extends AbstractMojo {
                     } else {
                         this.getLog().debug("Adding descriptor " + comp);
                         components.addComponent(comp);
+                        abstractComponents.addComponent(comp);
                     }
                 } else {
                     hasFailures = true;
@@ -125,9 +134,9 @@ public class SCRDescriptorMojo extends AbstractMojo {
             throw new MojoFailureException("SCR Descriptor parsing had failures (see log)");
         }
 
-        // write meta type info if there is a file
+        boolean addResources = false;
+        // write meta type info if there is a file name
         if (!StringUtils.isEmpty(this.metaTypeName)) {
-
             File mtFile = new File(this.outputDirectory, "OSGI-INF" + File.separator + "metatype" + File.separator + this.metaTypeName);
             if ( metaData.getDescriptors().size() > 0 ) {
                 this.getLog().info("Generating "
@@ -135,6 +144,7 @@ public class SCRDescriptorMojo extends AbstractMojo {
                     + " MetaType Descriptors to " + mtFile);
                 mtFile.getParentFile().mkdirs();
                 MetaTypeIO.write(metaData, mtFile);
+                addResources = true;
             } else {
                 if ( mtFile.exists() ) {
                     mtFile.delete();
@@ -142,15 +152,16 @@ public class SCRDescriptorMojo extends AbstractMojo {
             }
 
         } else {
-            this.getLog().info("Have no meta type file name, not writing metatype info");
+            this.getLog().info("Meta type file name is not set: meta type info is not written.");
         }
 
-        // if we have abstract descriptors, write them
+        // if we have descriptors, write them in our scr private file (for component inheritance)
         final File adFile = new File(this.outputDirectory, Constants.ABSTRACT_DESCRIPTOR_RELATIVE_PATH);
         if ( !abstractComponents.getComponents().isEmpty() ) {
             this.getLog().info("Writing abstract service descriptor " + adFile + " with " + abstractComponents.getComponents().size() + " entries.");
             adFile.getParentFile().mkdirs();
-            ComponentDescriptorIO.write(abstractComponents, adFile);
+            ComponentDescriptorIO.write(abstractComponents, adFile, true);
+            addResources = true;
         } else {
             this.getLog().debug("No abstract SCR Descriptors found in project.");
             // remove file
@@ -160,46 +171,53 @@ public class SCRDescriptorMojo extends AbstractMojo {
             }
         }
 
+        // check descriptor file
+        final File descriptorFile = StringUtils.isEmpty(this.finalName) ? null : new File(new File(this.outputDirectory, "OSGI-INF"), this.finalName);
+
         // terminate if there is nothing else to write
         if (components.getComponents().isEmpty()) {
             this.getLog().debug("No SCR Descriptors found in project.");
-            return;
+            // remove file if it exists
+            if ( descriptorFile != null && descriptorFile.exists() ) {
+                this.getLog().debug("Removing obsolete service descriptor " + descriptorFile);
+                descriptorFile.delete();
+            }
+        } else {
+
+            if (descriptorFile == null) {
+                throw new MojoFailureException("Descriptor file name must not be empty.");
+            }
+
+            // finally the descriptors have to be written ....
+            descriptorFile.getParentFile().mkdirs(); // ensure parent dir
+
+            this.getLog().info("Generating " + components.getComponents().size()
+                    + " Service Component Descriptors to " + descriptorFile);
+
+            ComponentDescriptorIO.write(components, descriptorFile, false);
+            addResources = true;
+
+            // and set include accordingly
+            String svcComp = project.getProperties().getProperty("Service-Component");
+            svcComp= (svcComp == null) ? "OSGI-INF/" + finalName : svcComp + ", " + "OSGI-INF/" + finalName;
+            project.getProperties().setProperty("Service-Component", svcComp);
         }
 
-        // check file name
-        if (StringUtils.isEmpty(this.finalName)) {
-            this.getLog().error("Descriptor file name must not be empty.");
-            return;
+        // now add the descriptor directory to the maven resources
+        if (addResources) {
+            final String ourRsrcPath = this.outputDirectory.getAbsolutePath();
+            boolean found = false;
+            final Iterator rsrcIterator = this.project.getResources().iterator();
+            while ( !found && rsrcIterator.hasNext() ) {
+                final Resource rsrc = (Resource)rsrcIterator.next();
+                found = rsrc.getDirectory().equals(ourRsrcPath);
+            }
+            if ( !found ) {
+                final Resource resource = new Resource();
+                resource.setDirectory(this.outputDirectory.getAbsolutePath());
+                this.project.addResource(resource);
+            }
         }
-
-        // finally the descriptors have to be written ....
-        File descriptorFile = new File(new File(this.outputDirectory, "OSGI-INF"), this.finalName);
-        descriptorFile.getParentFile().mkdirs(); // ensure parent dir
-
-        this.getLog().info("Generating " + components.getComponents().size()
-                + " Service Component Descriptors to " + descriptorFile);
-
-        ComponentDescriptorIO.write(components, descriptorFile);
-
-
-        // now add the descriptor file to the maven resources
-        final String ourRsrcPath = this.outputDirectory.getAbsolutePath();
-        boolean found = false;
-        final Iterator rsrcIterator = this.project.getResources().iterator();
-        while ( !found && rsrcIterator.hasNext() ) {
-            final Resource rsrc = (Resource)rsrcIterator.next();
-            found = rsrc.getDirectory().equals(ourRsrcPath);
-        }
-        if ( !found ) {
-            final Resource resource = new Resource();
-            resource.setDirectory(this.outputDirectory.getAbsolutePath());
-            this.project.addResource(resource);
-        }
-
-        // and set include accordingly
-        String svcComp = project.getProperties().getProperty("Service-Component");
-        svcComp= (svcComp == null) ? "OSGI-INF/" + finalName : svcComp + ", " + "OSGI-INF/" + finalName;
-        project.getProperties().setProperty("Service-Component", svcComp);
     }
 
     /**
@@ -218,19 +236,20 @@ public class SCRDescriptorMojo extends AbstractMojo {
 
         final OCD ocd = this.doComponent(componentTag, component, metaData);
 
-        boolean inherited = this.getBoolean(componentTag, Constants.COMPONENT_INHERIT, true);
+        boolean inherited = getBoolean(componentTag, Constants.COMPONENT_INHERIT, true);
         this.doServices(description.getTagsByName(Constants.SERVICE, inherited), component, description);
 
-        // collect properties and references from class tags and fields
-        final Map properties = new HashMap();
+        // collect references from class tags and fields
         final Map references = new HashMap();
+        // Utility handler for propertie
+        final PropertyHandler propertyHandler = new PropertyHandler(component, ocd);
 
         JavaClassDescription currentDescription = description;
         do {
             // properties
             final JavaTag[] props = currentDescription.getTagsByName(Constants.PROPERTY, false);
             for (int i=0; i < props.length; i++) {
-                this.testProperty(properties, props[i], null, description == currentDescription);
+                propertyHandler.testProperty(props[i], null, description == currentDescription);
             }
 
             // references
@@ -247,32 +266,14 @@ public class SCRDescriptorMojo extends AbstractMojo {
                     this.testReference(references, tag, fields[i].getName(), description == currentDescription);
                 }
 
-                tag = fields[i].getTagByName(Constants.PROPERTY);
-                if (tag != null) {
-                    String defaultName = null;
-                    if ( "java.lang.String".equals(fields[i].getType()) ) {
-                        defaultName = fields[i].getInitializationExpression().trim();
-                        int pos = defaultName.indexOf("\"");
-                        if ( pos != -1 ) {
-                            defaultName = defaultName.substring(pos + 1);
-                            defaultName = defaultName.substring(0, defaultName.lastIndexOf("\""));
-                        }
-                    }
-                    this.testProperty(properties, tag, defaultName, description == currentDescription);
-                }
+                propertyHandler.handleField(fields[i], description == currentDescription);
             }
 
             currentDescription = currentDescription.getSuperClass();
         } while (inherited && currentDescription != null);
 
         // process properties
-        final Iterator propIter = properties.entrySet().iterator();
-        while ( propIter.hasNext() ) {
-            final Map.Entry entry = (Map.Entry)propIter.next();
-            final String propName = entry.getKey().toString();
-            final JavaTag tag = (JavaTag)entry.getValue();
-            this.doProperty(tag, propName, component, ocd);
-        }
+        propertyHandler.processProperties();
 
         // process references
         final Iterator refIter = references.entrySet().iterator();
@@ -284,7 +285,7 @@ public class SCRDescriptorMojo extends AbstractMojo {
         }
 
         // pid handling
-        final boolean createPid = this.getBoolean(componentTag, Constants.COMPONENT_CREATE_PID, true);
+        final boolean createPid = getBoolean(componentTag, Constants.COMPONENT_CREATE_PID, true);
         if ( createPid ) {
             // check for an existing pid first
             boolean found = false;
@@ -342,9 +343,15 @@ public class SCRDescriptorMojo extends AbstractMojo {
         String name = tag.getNamedParameter(Constants.COMPONENT_NAME);
         component.setName(StringUtils.isEmpty(name) ? component.getImplementation().getClassame() : name);
 
-        component.setEnabled(Boolean.valueOf(this.getBoolean(tag, Constants.COMPONENT_ENABLED, true)));
+        component.setEnabled(Boolean.valueOf(getBoolean(tag, Constants.COMPONENT_ENABLED, true)));
         component.setFactory(tag.getNamedParameter(Constants.COMPONENT_FACTORY));
-        component.setImmediate(Boolean.valueOf(this.getBoolean(tag, Constants.COMPONENT_IMMEDIATE, true)));
+        
+        // FELIX-593: immediate attribute does not default to true all the
+        // times hence we only set it if declared in the tag
+        if (tag.getNamedParameter(Constants.COMPONENT_IMMEDIATE) != null) {
+            component.setImmediate(Boolean.valueOf(getBoolean(tag,
+                Constants.COMPONENT_IMMEDIATE, true)));
+        }
 
         // whether metatype information is to generated for the component
         final String metaType = tag.getNamedParameter(Constants.COMPONENT_METATYPE);
@@ -406,7 +413,7 @@ public class SCRDescriptorMojo extends AbstractMojo {
                 service.addInterface(interf);
             }
 
-            serviceFactory |= this.getBoolean(services[i], Constants.SERVICE_FACTORY, false);
+            serviceFactory |= getBoolean(services[i], Constants.SERVICE_FACTORY, false);
         }
 
         service.setServicefactory(serviceFactory);
@@ -432,134 +439,6 @@ public class SCRDescriptorMojo extends AbstractMojo {
         }
     }
 
-    /**
-     * @param property
-     * @param defaultName
-     * @param component
-     */
-    protected void doProperty(JavaTag property, String name, Component component, OCD ocd) {
-        final Property prop = new Property(property);
-        prop.setName(name);
-        prop.setType(property.getNamedParameter(Constants.PROPERTY_TYPE));
-        final String value = property.getNamedParameter(Constants.PROPERTY_VALUE);
-        if ( value != null ) {
-            prop.setValue(value);
-        } else {
-            // check for multivalue
-            final List values = new ArrayList();
-            final Map valueMap = property.getNamedParameterMap();
-            for (Iterator vi = valueMap.entrySet().iterator(); vi.hasNext();) {
-                final Map.Entry entry = (Map.Entry) vi.next();
-                final String key = (String) entry.getKey();
-                if (key.startsWith("values")) {
-                    values.add(entry.getValue());
-                }
-            }
-            if ( values.size() > 0 ) {
-                prop.setMultiValue((String[])values.toArray(new String[values.size()]));
-            }
-        }
-
-        // property is private if explicitly marked or a well known
-        // service property such as service.pid
-        final boolean isPrivate = getBoolean(property,
-            Constants.PROPERTY_PRIVATE, false)
-            || name.equals(org.osgi.framework.Constants.SERVICE_PID)
-            || name.equals(org.osgi.framework.Constants.SERVICE_DESCRIPTION)
-            || name.equals(org.osgi.framework.Constants.SERVICE_ID)
-            || name.equals(org.osgi.framework.Constants.SERVICE_RANKING)
-            || name.equals(org.osgi.framework.Constants.SERVICE_VENDOR)
-            || name.equals(ConfigurationAdmin.SERVICE_BUNDLELOCATION)
-            || name.equals(ConfigurationAdmin.SERVICE_FACTORYPID);
-
-        // if this is a public property and the component is generating metatype info
-        // store the information!
-        if ( !isPrivate && ocd != null ) {
-            final AttributeDefinition ad = new AttributeDefinition();
-            ocd.getProperties().add(ad);
-            ad.setId(prop.getName());
-            ad.setType(prop.getType());
-
-            String adName = property.getNamedParameter(Constants.PROPERTY_LABEL);
-            if ( adName == null ) {
-                adName = "%" + prop.getName() + ".name";
-            }
-            ad.setName(adName);
-            String adDesc = property.getNamedParameter(Constants.PROPERTY_DESCRIPTION);
-            if ( adDesc == null ) {
-                adDesc = "%" + prop.getName() + ".description";
-            }
-            ad.setDescription(adDesc);
-            // set optional multivalues, cardinality might be overwritten by setValues !!
-            final String cValue = property.getNamedParameter(Constants.PROPERTY_CARDINALITY);
-            if (cValue != null) {
-                if ("-".equals(cValue)) {
-                    // unlimited vector
-                    ad.setCardinality(new Integer(Integer.MIN_VALUE));
-                } else if ("+".equals(cValue)) {
-                   // unlimited array
-                    ad.setCardinality(new Integer(Integer.MAX_VALUE));
-                } else {
-                    try {
-                        ad.setCardinality(Integer.valueOf(cValue));
-                    } catch (NumberFormatException nfe) {
-                        // default to scalar in case of conversion problem
-                    }
-                }
-            }
-            ad.setDefaultValue(prop.getValue());
-            ad.setDefaultMultiValue(prop.getMultiValue());
-
-            // check options
-            String[] parameters = property.getParameters();
-            Map options = null;
-            for (int j=0; j < parameters.length; j++) {
-                if (Constants.PROPERTY_OPTIONS.equals(parameters[j])) {
-                    options = new LinkedHashMap();
-                } else if (options != null) {
-                    String optionLabel = parameters[j];
-                    String optionValue = (j < parameters.length-2) ? parameters[j+2] : null;
-                    if (optionValue != null) {
-                        options.put(optionLabel, optionValue);
-                    }
-                    j += 2;
-                }
-            }
-            ad.setOptions(options);
-        }
-
-        component.addProperty(prop);
-    }
-
-    protected String getPropertyName(JavaTag property, String defaultName) {
-        String name = property.getNamedParameter(Constants.PROPERTY_NAME);
-        if (StringUtils.isEmpty(name) && defaultName != null) {
-            name = defaultName;
-        }
-
-        if (!StringUtils.isEmpty(name)) {
-            return name;
-        }
-        return null;
-    }
-
-    protected void testProperty(Map properties, JavaTag property, String defaultName, boolean isInspectedClass)
-    throws MojoExecutionException {
-        final String propName = this.getPropertyName(property, defaultName);
-
-        if ( propName != null ) {
-            if ( properties.containsKey(propName) ) {
-                // if the current class is the class we are currently inspecting, we
-                // have found a duplicate definition
-                if ( isInspectedClass ) {
-                    throw new MojoExecutionException("Duplicate definition for property " + propName + " in class " + property.getJavaClassDescription().getName());
-                }
-            } else {
-                properties.put(propName, property);
-            }
-        }
-    }
-
     protected void testReference(Map references, JavaTag reference, String defaultName, boolean isInspectedClass)
     throws MojoExecutionException {
         final String refName = this.getReferenceName(reference, defaultName);
@@ -650,7 +529,7 @@ public class SCRDescriptorMojo extends AbstractMojo {
         component.addReference(ref);
     }
 
-    protected boolean getBoolean(JavaTag tag, String name, boolean defaultValue) {
+    public static boolean getBoolean(JavaTag tag, String name, boolean defaultValue) {
         String value = tag.getNamedParameter(name);
         return (value == null) ? defaultValue : Boolean.valueOf(value).booleanValue();
     }
