@@ -60,8 +60,22 @@ public interface JavaClassDescription {
      */
     String getName();
 
+    /**
+     * Get the fields of this class
+     * @return All fields or an empty array
+     */
     JavaField[] getFields();
 
+    /**
+     * Get the field with the name.
+     * @param name The name of the field
+     * @return The field with the name or null.
+     * @throws MojoExecutionException
+     */
+    JavaField getFieldByName(String name) throws MojoExecutionException;
+
+    JavaField getExternalFieldByName(String name) throws MojoExecutionException;
+
     /**
      * Returns an array of the implemented interfaces of this class.
      * @return An array containing the interfaces or an empty array
