@@ -18,21 +18,11 @@
  */
 package org.apache.felix.scrplugin.tags;
 
-import java.io.File;
-import java.io.FileOutputStream;
-import java.io.IOException;
+import java.io.*;
 import java.net.URL;
 import java.net.URLClassLoader;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.StringTokenizer;
-import java.util.jar.JarEntry;
-import java.util.jar.JarFile;
-import java.util.jar.Manifest;
+import java.util.*;
+import java.util.jar.*;
 
 import org.apache.felix.scrplugin.Constants;
 import org.apache.felix.scrplugin.om.Component;
@@ -46,7 +36,7 @@ import org.apache.maven.plugin.MojoExecutionException;
 import org.apache.maven.plugin.MojoFailureException;
 import org.apache.maven.plugin.logging.Log;
 import org.apache.maven.project.MavenProject;
-import org.codehaus.plexus.util.IOUtil;
+import org.codehaus.plexus.util.*;
 
 import com.thoughtworks.qdox.JavaDocBuilder;
 import com.thoughtworks.qdox.model.JavaSource;
@@ -84,22 +74,60 @@ public class JavaClassDescriptorManager {
      * @throws MojoFailureException
      * @throws MojoExecutionException
      */
-    public JavaClassDescriptorManager(final Log         log,
-                                      final MavenProject project)
+    public JavaClassDescriptorManager(final Log          log,
+                                      final MavenProject project,
+                                      final String       excludeString)
     throws MojoFailureException, MojoExecutionException {
         this.log = log;
         this.project = project;
-        this.classloader = this.getCompileClassLoader(project);
+        this.classloader = this.getCompileClassLoader();
 
         // get all the class sources through qdox
         this.log.debug("Setting up QDox");
         JavaDocBuilder builder = new JavaDocBuilder();
         builder.getClassLibrary().addClassLoader(this.classloader);
         final Iterator i = project.getCompileSourceRoots().iterator();
-        while ( i.hasNext() ) {
-            final String tree = (String)i.next();
-            this.log.debug("Adding source tree " + tree);
-            builder.addSourceTree(new File(tree));
+        // FELIX-509: check for excludes
+        if ( excludeString != null ) {
+            final String[] excludes = StringUtils.split(excludeString, ",");
+            final String[] includes = new String[] {"**/*.java"};
+
+            while ( i.hasNext() ) {
+                final String tree = (String)i.next();
+                this.log.debug("Scanning source tree " + tree);
+                final File directory = new File(tree);
+                final DirectoryScanner scanner = new DirectoryScanner();
+                scanner.setBasedir( directory );
+
+                if ( excludes != null && excludes.length > 0 ) {
+                    scanner.setExcludes( excludes );
+                }
+                scanner.addDefaultExcludes();
+                scanner.setIncludes( includes );
+
+                scanner.scan();
+
+                final String[] files = scanner.getIncludedFiles();
+                if ( files != null ) {
+                    for(int m=0; m<files.length; m++) {
+                        this.log.debug("Adding source file " + files[m]);
+                        try {
+                            builder.addSource(new File(directory, files[m]));
+                        } catch (FileNotFoundException e) {
+                            throw new MojoExecutionException("Unable to scan directory.", e);
+                        } catch (IOException e) {
+                            throw new MojoExecutionException("Unable to scan directory.", e);
+                        }
+                    }
+                }
+            }
+        } else {
+            while ( i.hasNext() ) {
+                final String tree = (String)i.next();
+                this.log.debug("Adding source tree " + tree);
+                final File directory = new File(tree);
+                builder.addSourceTree(directory);
+            }
         }
         this.sources = builder.getSources();
 
@@ -141,6 +169,8 @@ public class JavaClassDescriptorManager {
                         throw new MojoExecutionException("Unable to get manifest from artifact " + artifact, ioe);
                     }
                     this.log.debug("Trying to get scrinfo from artifact " + artifact);
+                    // now read the scr private file - components stored there overwrite components already
+                    // read from the service component section.
                     try {
                         final File scrInfoFile = this.getFile(artifact, Constants.ABSTRACT_DESCRIPTOR_ARCHIV_PATH);
                         if ( scrInfoFile != null ) {
@@ -211,10 +241,10 @@ public class JavaClassDescriptorManager {
         return list;
     }
 
-    protected ClassLoader getCompileClassLoader(MavenProject project)
+    protected ClassLoader getCompileClassLoader()
     throws MojoFailureException {
-        List artifacts = project.getCompileArtifacts();
-        URL[] path = new URL[artifacts.size()];
+        List artifacts = this.getProject().getCompileArtifacts();
+        URL[] path = new URL[artifacts.size() + 1];
         int i = 0;
         for (Iterator ai=artifacts.iterator(); ai.hasNext(); ) {
             Artifact a = (Artifact) ai.next();
@@ -224,7 +254,13 @@ public class JavaClassDescriptorManager {
                 throw new MojoFailureException("Unable to get compile class loader.");
             }
         }
-        return new URLClassLoader(path);
+        final String targetDirectory = this.getProject().getBuild().getOutputDirectory();
+        try {
+            path[path.length - 1] = new File(targetDirectory).toURI().toURL();
+        } catch (IOException ioe) {
+            throw new MojoFailureException("Unable to add target directory to classloader.");
+        }
+        return new URLClassLoader(path, this.getClass().getClassLoader());
     }
 
     protected Manifest getManifest(Artifact artifact) throws IOException {
@@ -273,10 +309,15 @@ public class JavaClassDescriptorManager {
      * Return all source descriptions of this project.
      * @return All contained java class descriptions.
      */
-    public JavaClassDescription[] getSourceDescriptions() {
+    public JavaClassDescription[] getSourceDescriptions() throws MojoExecutionException {
         final JavaClassDescription[] descs = new JavaClassDescription[this.sources.length];
         for(int i=0; i<this.sources.length; i++) {
-            descs[i] = new QDoxJavaClassDescription(this.sources[i], this);
+            final String className = this.sources[i].getClasses()[0].getFullyQualifiedName();
+            try {
+                descs[i] = new QDoxJavaClassDescription(this.classloader.loadClass(className), this.sources[i], this);
+            } catch (ClassNotFoundException e) {
+                throw new MojoExecutionException("Unable to load class " + className);
+            }
         }
         return descs;
     }
@@ -295,8 +336,12 @@ public class JavaClassDescriptorManager {
             int index = 0;
             while ( result == null && index < this.sources.length) {
                 if ( this.sources[index].getClasses()[0].getFullyQualifiedName().equals(className) ) {
-                    this.log.debug("Found qdox description for: " + className);
-                    result = new QDoxJavaClassDescription(this.sources[index], this);
+                    try {
+                        this.log.debug("Found qdox description for: " + className);
+                        result = new QDoxJavaClassDescription(this.classloader.loadClass(className), this.sources[index], this);
+                    } catch (ClassNotFoundException e) {
+                        throw new MojoExecutionException("Unable to load class " + className);
+                    }
                 } else {
                     index++;
                 }
